/*
 * Copyright (c) 2008-2013, Hazelcast, Inc. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.hazelcast.spi.impl;

import com.hazelcast.core.MemberLeftException;
import com.hazelcast.instance.MemberImpl;
import com.hazelcast.nio.Address;
import com.hazelcast.spi.Callback;

final class RemoteCall {

    private final Address target;
    private final String uuid;
    private final Callback<Object> callback;

    RemoteCall(Address target, Callback<Object> callback) {
        this.target = target;
        this.callback = callback;
        this.uuid = null;
    }

    RemoteCall(MemberImpl target, Callback<Object> callback) {
        this.target = target.getAddress();
        this.uuid = target.getUuid();
        this.callback = callback;
    }

    void offerResponse(Object response) {
        callback.notify(response);
    }

    void onMemberLeft(MemberImpl leftMember) {
        boolean notify;
        if (uuid != null) {
            notify = leftMember.getUuid().equals(uuid);
        } else {
            notify = leftMember.getAddress().equals(target);
        }

        if (notify) {
            callback.notify(new MemberLeftException(leftMember));
        }
    }

    @Override
    public String toString() {
        return "RemoteCall{" +
               "target=" + target +
               ", callback=" + callback +
               '}';
    }
}
